@@ -24,6 +24,7 @@ import click
 
 from klio_core import options as core_options
 from klio_core import utils as core_utils
+from klio_core import variables as var
 
 from klio_cli import __version__ as version
 from klio_cli import options
@@ -54,6 +55,7 @@ logging.getLogger().setLevel(logging.INFO)
 DockerRuntimeConfig = collections.namedtuple(
     "DockerRuntimeConfig", ["image_tag", "force_build", "config_file_override"]
 )
+# TODO: We can get rid of direct_runner since we have runner
 RunJobConfig = collections.namedtuple(
     "RunJobConfig", ["direct_runner", "update", "git_sha"]
 )
@@ -134,6 +136,7 @@ def run_job(klio_config, config_meta, **kwargs):
     direct_runner = cli_utils.is_direct_runner(
         klio_config, kwargs.pop("direct_runner")
     )
+    cli_utils.error_stackdriver_logger_metrics(klio_config, direct_runner)
 
     git_sha = cli_utils.get_git_sha(
         config_meta.job_dir, kwargs.get("image_tag")
@@ -151,9 +154,21 @@ def run_job(klio_config, config_meta, **kwargs):
         update=kwargs.pop("update"),
         git_sha=git_sha,
     )
-    klio_pipeline = job_commands.run.RunPipeline(
-        config_meta.job_dir, klio_config, runtime_config, run_job_config
-    )
+
+    if (
+        not direct_runner
+        and klio_config.pipeline_options.runner
+        == var.KlioRunner.DIRECT_GKE_RUNNER
+    ):
+        gke_commands = cli_utils.import_gke_commands()
+
+        klio_pipeline = gke_commands.RunPipelineGKE(
+            config_meta.job_dir, klio_config, runtime_config, run_job_config
+        )
+    else:
+        klio_pipeline = job_commands.run.RunPipeline(
+            config_meta.job_dir, klio_config, runtime_config, run_job_config
+        )
     rc = klio_pipeline.run()
     sys.exit(rc)
 
@@ -161,8 +176,7 @@ def run_job(klio_config, config_meta, **kwargs):
 @job.command(
     "stop",
     help=(
-        "Cancel a currently running job.\n\n**NOTE:** Draining is not "
-        "supported."
+        "Cancel a currently running job.\n\n**NOTE:** Draining is not supported"
     ),
 )
 @core_options.job_dir
@@ -203,7 +217,14 @@ def stop_job(klio_config, config_meta, job_name, region, gcp_project):
 
     # TODO: make this a click option once draining is supported @lynn
     strategy = "cancel"
-    job_commands.stop.StopJob().stop(job_name, gcp_project, region, strategy)
+    if klio_config.pipeline_options.runner == var.KlioRunner.DIRECT_GKE_RUNNER:
+        gke_commands = cli_utils.import_gke_commands()
+
+        gke_commands.StopPipelineGKE(config_meta.job_dir).stop()
+    else:
+        job_commands.stop.StopJob().stop(
+            job_name, gcp_project, region, strategy
+        )
 
 
 @job.command(
@@ -220,6 +241,7 @@ def deploy_job(klio_config, config_meta, **kwargs):
     direct_runner = cli_utils.is_direct_runner(
         klio_config, kwargs.pop("direct_runner")
     )
+    cli_utils.error_stackdriver_logger_metrics(klio_config, direct_runner)
 
     git_sha = cli_utils.get_git_sha(
         config_meta.job_dir, kwargs.get("image_tag")
@@ -250,10 +272,21 @@ def deploy_job(klio_config, config_meta, **kwargs):
             job_name, gcp_project, region, strategy
         )
 
-    klio_pipeline = job_commands.run.RunPipeline(
-        config_meta.job_dir, klio_config, runtime_config, run_job_config
-    )
-    rc = klio_pipeline.run()
+    if (
+        not direct_runner
+        and klio_config.pipeline_options.runner
+        == var.KlioRunner.DIRECT_GKE_RUNNER
+    ):
+        gke_commands = cli_utils.import_gke_commands()
+
+        run_command = gke_commands.RunPipelineGKE(
+            config_meta.job_dir, klio_config, runtime_config, run_job_config
+        )
+    else:
+        run_command = job_commands.run.RunPipeline(
+            config_meta.job_dir, klio_config, runtime_config, run_job_config
+        )
+    rc = run_command.run()
     sys.exit(rc)
 
 
@@ -299,7 +332,12 @@ def create_job(addl_job_opts, output, **known_kwargs):
 )
 @core_utils.with_klio_config
 def delete_job(klio_config, config_meta):
-    job_commands.delete.DeleteJob(klio_config).delete()
+    if klio_config.pipeline_options.runner == var.KlioRunner.DIRECT_GKE_RUNNER:
+        gke_commands = cli_utils.import_gke_commands()
+
+        gke_commands.DeletePipelineGKE(config_meta.job_dir).delete()
+    else:
+        job_commands.delete.DeleteJob(klio_config).delete()
 
 
 @job.command(
@@ -315,8 +353,7 @@ def delete_job(klio_config, config_meta):
 def test_job(
     klio_config, config_meta, force_build, image_tag, pytest_args, **kwargs
 ):
-    """Thin wrapper around pytest. Any arguments after -- are passed through.
-    """
+    """Thin wrapper around pytest. Any arguments after -- are passed through."""
     pytest_args = list(pytest_args)
 
     if pytest_args:
