@@ -24,15 +24,38 @@ from google.cloud import storage
 from googleapiclient import discovery
 from googleapiclient import errors as google_errors
 
+from klio_core import variables as var
+
 from klio_cli.utils import stackdriver_utils as sd_utils
 
 
 GCP_ROLE_METRIC_WRITER = "roles/monitoring.metricWriter"
+GCP_ROLE_PUBLISHER = "roles/pubsub.publisher"
+GCP_ROLE_SUBSCRIBER = "roles/pubsub.subscriber"
+GCP_ROLE_STORAGE_CREATOR = "roles/storage.objectCreator"
+GCP_ROLE_STORAGE_VIEWER = "roles/storage.objectViewer"
+GCP_ROLE_LOG_WRITER = "roles/logging.logWriter"
+GCP_ROLE_SERV_ACCT_USER = "roles/iam.serviceAccountUser"
 #
 # Add any extra roles we need to check on the default service account
 # to this set:
 #
-ROLES_TO_CHECK = {GCP_ROLE_METRIC_WRITER}
+BASE_ROLES_TO_CHECK = {
+    GCP_ROLE_METRIC_WRITER,
+    GCP_ROLE_STORAGE_CREATOR,
+    GCP_ROLE_STORAGE_VIEWER,
+}
+STREAMING_ROLES_TO_CHECK = {
+    *BASE_ROLES_TO_CHECK,
+    GCP_ROLE_PUBLISHER,
+    GCP_ROLE_SUBSCRIBER,
+}
+DIRECT_GKE_ROLES_TO_CHECK = {
+    *STREAMING_ROLES_TO_CHECK,
+    GCP_ROLE_LOG_WRITER,
+    GCP_ROLE_SERV_ACCT_USER,
+}
+
 
 logging.getLogger("googleapiclient.discovery").setLevel(logging.ERROR)
 
@@ -158,9 +181,9 @@ class VerifyJob(object):
         logging.info("Verifying your {} topic: {}".format(_type, topic))
         try:
             if self.create_resources:
-                self.publisher_client.create_topic(topic)
+                self.publisher_client.create_topic(request={"name": topic})
             else:
-                self.publisher_client.get_topic(topic)
+                self.publisher_client.get_topic(request={"topic": topic})
                 logging.info("Topic {} exists".format(topic))
             return True
         except api_ex.AlreadyExists:
@@ -199,7 +222,7 @@ class VerifyJob(object):
             try:
                 if self.create_resources:
                     self.subscriber_client.create_subscription(
-                        name=sub, topic=upstream_topic
+                        request={"name": sub, "topic": upstream_topic}
                     )
                     logging.info(
                         "Creating the subscription {} to topic {}".format(
@@ -208,7 +231,9 @@ class VerifyJob(object):
                     )
                     logging.info("Successfully created your subscription.")
                 else:
-                    self.subscriber_client.get_subscription(sub)
+                    self.subscriber_client.get_subscription(
+                        request={"subscription": sub}
+                    )
                     logging.info("Subscription %s exists" % sub)
             except api_ex.AlreadyExists:
                 logging.info("Subscription {} exists".format(sub))
@@ -255,7 +280,10 @@ class VerifyJob(object):
 
         logging.info("Verifying your event inputs...")
 
-        for _input in event_inputs:
+        pubsub_event_inputs = [
+            _i for _i in event_inputs if _i.name == "pubsub"
+        ]
+        for _input in pubsub_event_inputs:
             input_topic = _input.topic
             input_subscription = _input.subscription
 
@@ -277,14 +305,21 @@ class VerifyJob(object):
                     )
                 )
 
-        logging.info(
-            "You have {} unverified input buckets, {} unverified input "
-            "topics and {} unverified input subscriptions".format(
-                unverified_bucket_count,
-                unverified_topic_count,
-                unverified_sub_count,
+        if len(pubsub_event_inputs):
+            logging.info(
+                "You have {} unverified input buckets, {} unverified input "
+                "topics and {} unverified input subscriptions".format(
+                    unverified_bucket_count,
+                    unverified_topic_count,
+                    unverified_sub_count,
+                )
+            )
+        else:
+            logging.info(
+                "You have {} unverified input buckets".format(
+                    unverified_bucket_count,
+                )
             )
-        )
 
         if (
             unverified_bucket_count
@@ -319,7 +354,10 @@ class VerifyJob(object):
 
         logging.info("Verifying your event outputs...")
 
-        for output in event_outputs:
+        pubsub_event_outputs = [
+            _i for _i in event_outputs if _i.name == "pubsub"
+        ]
+        for output in pubsub_event_outputs:
             output_topic = output.topic
 
             if output_topic is not None:
@@ -329,11 +367,20 @@ class VerifyJob(object):
             else:
                 logging.error("There is no topic for {}".format(output))
 
-        logging.info(
-            "You have {} unverified output buckets and {} unverified output "
-            "topics".format(unverified_bucket_count, unverified_topic_count)
-        )
-
+        if pubsub_event_outputs:
+            logging.info(
+                "You have {} unverified output buckets "
+                "and {} unverified output "
+                "topics".format(
+                    unverified_bucket_count, unverified_topic_count
+                )
+            )
+        else:
+            logging.info(
+                "You have {} unverified output buckets".format(
+                    unverified_bucket_count
+                )
+            )
         if unverified_bucket_count + unverified_topic_count == 0:
             return True
         return False
@@ -403,22 +450,42 @@ class VerifyJob(object):
                 "unsafe project editor or owner permissions."
             )
 
-        if ROLES_TO_CHECK.issubset(svc_account_roles):
+        runner = self.klio_config.pipeline_options.runner
+        if runner == var.KlioRunner.DIRECT_GKE_RUNNER:
+            roles_to_check = DIRECT_GKE_ROLES_TO_CHECK
+        if self.klio_config.pipeline_options.streaming:
+            roles_to_check = STREAMING_ROLES_TO_CHECK
+        else:
+            roles_to_check = BASE_ROLES_TO_CHECK
+
+        if roles_to_check.issubset(svc_account_roles):
             logging.info(
                 "Verified that the service account has the required roles"
             )
             return True
 
-        logging.error(
-            "The default compute service account is missing the following IAM "
-            "roles: {}".format(ROLES_TO_CHECK - svc_account_roles)
+        missing_roles = roles_to_check - svc_account_roles
+        logging.warning(
+            "The configured compute service account is missing the following "
+            f"IAM role(s): {', '.join(missing_roles)}"
         )
         if self.create_resources:
-            logging.error(
-                "--create-resources is not able to add"
-                " these roles to the service "
-                "account at this time. Please add them manually via the Google "
-                "Cloud console."
+            gcloud_cmd_base = (
+                f"\tgcloud projects add-iam-policy-binding {self.project} \\\n"
+                f"\t\t--member={svc_account_string} \\\n"
+                "\t\t--role="
+            )
+            gcloud_cmds = []
+            for role in missing_roles:
+                cmd = gcloud_cmd_base + role
+                gcloud_cmds.append(cmd)
+
+            gcloud_cmds = "\n".join(gcloud_cmds)
+            logging.warning(
+                "Klio is unable to add the required role(s) to the service "
+                "account at this time. Add them with the following gcloud "
+                "command(s):\n"
+                f"{gcloud_cmds}"
             )
         return False
 
