@@ -15,6 +15,7 @@
 
 import logging
 
+import httplib2
 import pytest
 
 from google.api_core import exceptions as api_ex
@@ -126,7 +127,23 @@ def mock_create_sd_group(mocker):
                 {
                     "role": "roles/monitoring.metricWriter",
                     "members": ["serviceAccount:the-default-svc-account"],
-                }
+                },
+                {
+                    "role": "roles/pubsub.publisher",
+                    "members": ["serviceAccount:the-default-svc-account"],
+                },
+                {
+                    "role": "roles/pubsub.subscriber",
+                    "members": ["serviceAccount:the-default-svc-account"],
+                },
+                {
+                    "role": "roles/storage.objectCreator",
+                    "members": ["serviceAccount:the-default-svc-account"],
+                },
+                {
+                    "role": "roles/storage.objectViewer",
+                    "members": ["serviceAccount:the-default-svc-account"],
+                },
             ],
             False,
         ),
@@ -184,7 +201,7 @@ def test_verify_iam_roles(
     result = job._verify_iam_roles()
     if create_resources:
         assert (
-            "--create-resources is not able to add these roles"
+            "Klio is unable to add the required role(s)"
             in caplog.records[-1].msg
         )
 
@@ -203,6 +220,22 @@ def test_verify_iam_roles_editor(caplog, klio_config, mock_discovery_client):
             "role": "roles/monitoring.metricWriter",
             "members": ["serviceAccount:the-default-svc-account"],
         },
+        {
+            "role": "roles/pubsub.publisher",
+            "members": ["serviceAccount:the-default-svc-account"],
+        },
+        {
+            "role": "roles/pubsub.subscriber",
+            "members": ["serviceAccount:the-default-svc-account"],
+        },
+        {
+            "role": "roles/storage.objectCreator",
+            "members": ["serviceAccount:the-default-svc-account"],
+        },
+        {
+            "role": "roles/storage.objectViewer",
+            "members": ["serviceAccount:the-default-svc-account"],
+        },
         {
             "role": "roles/editor",
             "members": ["serviceAccount:the-default-svc-account"],
@@ -247,6 +280,22 @@ def test_verify_iam_roles_with_svc_account(klio_config, mock_discovery_client):
             "role": "roles/monitoring.metricWriter",
             "members": ["serviceAccount:my.sa@something.com"],
         },
+        {
+            "role": "roles/pubsub.publisher",
+            "members": ["serviceAccount:my.sa@something.com"],
+        },
+        {
+            "role": "roles/pubsub.subscriber",
+            "members": ["serviceAccount:my.sa@something.com"],
+        },
+        {
+            "role": "roles/storage.objectCreator",
+            "members": ["serviceAccount:my.sa@something.com"],
+        },
+        {
+            "role": "roles/storage.objectViewer",
+            "members": ["serviceAccount:my.sa@something.com"],
+        },
         {
             "role": "roles/editor",
             "members": ["serviceAccount:the-default-svc-account"],
@@ -279,8 +328,10 @@ def test_verify_iam_roles_with_svc_account(klio_config, mock_discovery_client):
 def test_verify_iam_roles_http_error(klio_config, mock_discovery_client):
     compute_client = mock_discovery_client.build("compute")
     err = google_errors.HttpError
+    resp = httplib2.Response({})
+    resp.reason = "some resp"
     compute_client.projects().get().execute.side_effect = err(
-        "some resp", "some content".encode()
+        resp, "some content".encode()
     )
     iam_client = mock_discovery_client.build("cloudresourcemanager")
     job = verify.VerifyJob(klio_config, False)
@@ -298,7 +349,7 @@ def test_verify_iam_roles_http_error(klio_config, mock_discovery_client):
     iam_client.projects().getIamPolicy(
         resource=job.klio_config.pipeline_options.project, body={}
     ).execute.side_effect = google_errors.HttpError(
-        "some resp", "some content".encode()
+        resp, "some content".encode()
     )
     job._compute_client = compute_client
     job._iam_client = iam_client
@@ -420,9 +471,13 @@ def test_verify_pub_topic(klio_config, mock_publisher, create_resources):
     actual = job._verify_pub_topic(test_topic, input)
 
     if create_resources:
-        mock_publisher.create_topic.assert_called_once_with(test_topic)
+        mock_publisher.create_topic.assert_called_once_with(
+            request={"name": test_topic}
+        )
     else:
-        mock_publisher.get_topic.assert_called_once_with(test_topic)
+        mock_publisher.get_topic.assert_called_once_with(
+            request={"topic": test_topic}
+        )
 
     assert actual is True
 
@@ -469,11 +524,13 @@ def test_verify_subscription_and_topic(
     if create_resources:
         actual = job._verify_subscription_and_topic(test_sub, upstream_topic,)
         mock_sub.create_subscription.assert_called_once_with(
-            name=test_sub, topic=upstream_topic
+            request={"name": test_sub, "topic": upstream_topic}
         )
     else:
         actual = job._verify_subscription_and_topic(test_sub, upstream_topic,)
-        mock_sub.get_subscription.assert_called_once_with(test_sub)
+        mock_sub.get_subscription.assert_called_once_with(
+            request={"subscription": test_sub}
+        )
 
     expected = True, True
 
@@ -524,6 +581,83 @@ def test_verify_subscription_and_topic_exceptions(
     assert expected == actual
 
 
+mock_gcs_event_config = {
+    "type": "gcs",
+    "location": "gs://some/events.txt",
+    "io_type": kconfig._io.KlioIOType.EVENT,
+    "io_direction": kconfig._io.KlioIODirection.INPUT,
+}
+mock_gcs_event = kconfig._io.KlioReadFileConfig.from_dict(
+    mock_gcs_event_config
+)
+
+mock_bq_event_config = {
+    "type": "bq",
+    "project": "sigint",
+    "dataset": "test-data",
+    "table": "test-table",
+    "io_type": kconfig._io.KlioIOType.EVENT,
+    "io_direction": kconfig._io.KlioIODirection.INPUT,
+}
+mock_bq_event = kconfig._io.KlioBigQueryEventInput.from_dict(
+    mock_bq_event_config
+)
+
+mock_avro_event_config = {
+    "type": "avro",
+    "io_type": kconfig._io.KlioIOType.EVENT,
+    "io_direction": kconfig._io.KlioIODirection.INPUT,
+}
+mock_avro_event = kconfig._io.KlioReadAvroEventConfig.from_dict(
+    mock_avro_event_config
+)
+
+
+@pytest.mark.parametrize(
+    "mock_event_input", ((mock_gcs_event), (mock_bq_event), (mock_avro_event),)
+)
+def test_unverified_event_inputs(
+    mocker,
+    caplog,
+    mock_storage,
+    mock_publisher,
+    mock_sub,
+    klio_config,
+    mock_event_input,
+):
+    mock_verify_gcs_bucket = mocker.patch.object(
+        verify.VerifyJob, "_verify_gcs_bucket"
+    )
+    mock_verify_sub = mocker.patch.object(
+        verify.VerifyJob, "_verify_subscription_and_topic"
+    )
+    job = verify.VerifyJob(klio_config, False)
+    job._publisher_client = mock_publisher
+    job._storage_client = mock_storage
+    job._subscriber_client = mock_sub
+    job.klio_config.pipeline_options.project = "sigint"
+
+    job.klio_config.job_config.events.inputs = [mock_event_input]
+
+    data_config = {
+        "type": "gcs",
+        "location": "test",
+        "io_type": kconfig._io.KlioIOType.DATA,
+        "io_direction": kconfig._io.KlioIODirection.INPUT,
+    }
+    job.klio_config.job_config.data.inputs = [
+        kconfig._io.KlioGCSInputDataConfig.from_dict(data_config)
+    ]
+    mock_verify_gcs_bucket.return_value = True
+
+    actual = job._verify_inputs()
+
+    mock_verify_sub.assert_not_called()
+    mock_verify_gcs_bucket.assert_called_with("test")
+    assert actual
+    assert 3 == len(caplog.records)
+
+
 @pytest.mark.parametrize(
     "unverified_bucket, unverified_topic, unverified_sub",
     (
@@ -674,6 +808,73 @@ def test_verify_inputs_logs(
     assert expected_log_count == len(caplog.records)
 
 
+mock_gcs_output_event_config = {
+    "type": "gcs",
+    "location": "gs://some/events.txt",
+    "io_type": kconfig._io.KlioIOType.EVENT,
+    "io_direction": kconfig._io.KlioIODirection.OUTPUT,
+}
+mock_gcs_output_event = kconfig._io.KlioWriteFileConfig.from_dict(
+    mock_gcs_event_config
+)
+mock_bq_output_event_config = {
+    "type": "bq",
+    "project": "sigint",
+    "dataset": "test-data",
+    "table": "test-table",
+    "schema": {"fields": [{"name": "n", "type": "t", "mode": "m"}]},
+    "io_type": kconfig._io.KlioIOType.EVENT,
+    "io_direction": kconfig._io.KlioIODirection.OUTPUT,
+}
+
+mock_bq_output_event = kconfig._io.KlioBigQueryEventOutput.from_dict(
+    mock_bq_output_event_config
+)
+
+
+@pytest.mark.parametrize(
+    "mock_event_output", ((mock_gcs_output_event), (mock_bq_output_event),)
+)
+def test_unverified_event_outputs(
+    mocker,
+    caplog,
+    klio_config,
+    mock_event_output,
+    mock_storage,
+    mock_publisher,
+):
+    mock_verify_gcs_bucket = mocker.patch.object(
+        verify.VerifyJob, "_verify_gcs_bucket"
+    )
+    mock_verify_pub_topic = mocker.patch.object(
+        verify.VerifyJob, "_verify_pub_topic"
+    )
+    job = verify.VerifyJob(klio_config, False)
+    job._publisher_client = mock_publisher
+    job._storage_client = mock_storage
+
+    job.klio_config.job_config.events.outputs = [mock_event_output]
+
+    data_config = {
+        "type": "gcs",
+        "location": "test",
+        "io_type": kconfig._io.KlioIOType.DATA,
+        "io_direction": kconfig._io.KlioIODirection.OUTPUT,
+    }
+    job.klio_config.job_config.data.outputs = [
+        kconfig._io.KlioGCSOutputDataConfig.from_dict(data_config)
+    ]
+
+    mock_verify_gcs_bucket.return_value = True
+
+    actual = job._verify_outputs()
+
+    assert actual
+    mock_verify_pub_topic.assert_not_called()
+    mock_verify_gcs_bucket.assert_called_with("test")
+    assert 3 == len(caplog.records)
+
+
 @pytest.mark.parametrize(
     "unverified_gcs, unverified_topic",
     ((False, False), (True, False), (True, True), (False, True)),
