@@ -21,84 +21,20 @@ import pytest
 
 from click import testing
 
+from klio_core import _testing as core_testing
 from klio_core import config as kconfig
+from klio_core import utils as core_utils
 
 from klio_cli import cli
+from klio_cli.commands.job import gke as gke_commands
 from klio_cli.utils import cli_utils
-from klio_cli.utils import config_utils
-
-
-def assert_execution_success(result):
-    """Helper for testing CLI commands that emits errors if execution failed"""
-    if result.exception:
-        if result.stdout:
-            print("captured stdout: {}".format(result.stdout))
-        raise result.exception
-
-    assert 0 == result.exit_code
-
-
-class MockKlioConfig(object):
-    def __init__(self, mocker, monkeypatch, patch_os_getcwd):
-
-        self.mock_get_config = mocker.patch.object(
-            config_utils, "get_config_by_path"
-        )
-        self.mock_get_config_job_dir = mocker.Mock()
-        monkeypatch.setattr(
-            cli.cli_utils, "get_config_job_dir", self.mock_get_config_job_dir
-        )
-
-        self.patch_os_getcwd = patch_os_getcwd
-        self.mocker = mocker
-        self.monkeypatch = monkeypatch
-
-    def setup(self, config_data, config_file, config_override=None):
-        self.config_override = config_override
-        self.config_data = config_data
-        self.config_file = config_file
-
-        self.mock_warn_if_py2_job = self.mocker.Mock()
-        self.monkeypatch.setattr(
-            cli.cli_utils, "warn_if_py2_job", self.mock_warn_if_py2_job
-        )
-
-        self.mock_get_config_job_dir.return_value = (
-            self.patch_os_getcwd,
-            config_override or config_file,
-        )
-
-        self.mock_get_config.return_value = config_data
-
-        self.meta = cli_utils.KlioConfigMeta(
-            job_dir=self.patch_os_getcwd,
-            config_file=config_override,
-            config_path=config_override or config_file,
-        )
-
-        self.klio_config = kconfig.KlioConfig(config_data)
-
-        self.mock_klio_config = self.mocker.patch.object(
-            cli_utils.config, "KlioConfig"
-        )
-        self.mock_klio_config.return_value = self.klio_config
-
-        return self.klio_config
-
-    def assert_calls(self):
-        self.mock_get_config_job_dir.assert_called_once_with(
-            None, self.config_override
-        )
-        self.mock_get_config.assert_called_once_with(
-            self.config_override or self.config_file
-        )
-        self.mock_klio_config.assert_called_once_with(self.config_data)
-        self.mock_warn_if_py2_job.assert_called_once_with(self.patch_os_getcwd)
 
 
 @pytest.fixture
 def mock_klio_config(mocker, monkeypatch, patch_os_getcwd):
-    return MockKlioConfig(mocker, monkeypatch, patch_os_getcwd)
+    return core_testing.MockKlioConfig(
+        cli, mocker, monkeypatch, patch_os_getcwd
+    )
 
 
 @pytest.fixture
@@ -133,14 +69,23 @@ def config_file(patch_os_getcwd):
 @pytest.fixture
 def mock_warn_if_py2_job(mocker, monkeypatch):
     mock_warn = mocker.Mock()
-    monkeypatch.setattr(cli.cli_utils, "warn_if_py2_job", mock_warn)
+    monkeypatch.setattr(cli.core_utils, "warn_if_py2_job", mock_warn)
     return mock_warn
 
 
+@pytest.fixture
+def mock_error_stackdriver_logger_metrics(mocker, monkeypatch):
+    mock_error = mocker.Mock()
+    monkeypatch.setattr(
+        cli_utils, "error_stackdriver_logger_metrics", mock_error
+    )
+    return mock_error
+
+
 @pytest.fixture
 def mock_get_config_job_dir(mocker, monkeypatch):
     mock = mocker.Mock()
-    monkeypatch.setattr(cli.cli_utils, "get_config_job_dir", mock)
+    monkeypatch.setattr(cli.core_utils, "get_config_job_dir", mock)
     return mock
 
 
@@ -238,7 +183,7 @@ def test_build_image(
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     if image_tag:
@@ -270,7 +215,7 @@ def test_create_job(runner, mock_create, patch_os_getcwd):
         "--use-defaults",
     ]
     result = runner.invoke(cli.main, cli_inputs)
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     known_kwargs = {
@@ -286,7 +231,7 @@ def test_create_job_prompts(runner, mock_create, patch_os_getcwd):
     prompt_inputs = ["test-job", "test-gcp-project"]
     inputs = "\n".join(prompt_inputs)
     result = runner.invoke(cli.main, cli_inputs, input=inputs)
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     exp_output = (
         "Name of your new job: {0}\n"
@@ -316,7 +261,7 @@ def test_create_job_unknown_args(runner, mock_create, patch_os_getcwd):
         "test-input-topic",
     ]
     result = runner.invoke(cli.main, cli_inputs)
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     unknown_args = ("--input-topic", "test-input-topic")
@@ -367,7 +312,7 @@ def test_delete_job(
     if config_override:
         cli_inputs.extend(["--config-file", config_override])
     result = runner.invoke(cli.main, cli_inputs)
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     mock_klio_config.assert_calls()
@@ -375,6 +320,35 @@ def test_delete_job(
     mock_delete.return_value.delete.assert_called_once_with()
 
 
+def test_delete_job_gke(
+    mocker, monkeypatch, runner, patch_os_getcwd, mock_klio_config,
+):
+
+    config = {
+        "job_name": "test-job",
+        "version": 1,
+        "pipeline_options": {
+            "project": "test-project",
+            "runner": "DirectGKERunner",
+        },
+        "job_config": {},
+    }
+    mock_klio_config.setup(config, None, None)
+
+    mock_delete_gke = mocker.patch.object(
+        cli.job_commands.gke, "DeletePipelineGKE"
+    )
+
+    cli_inputs = ["job", "delete"]
+    result = runner.invoke(cli.main, cli_inputs)
+    core_testing.assert_execution_success(result)
+    assert "" == result.output
+
+    mock_klio_config.assert_calls()
+    mock_delete_gke.assert_called_once_with(mock_klio_config.meta.job_dir)
+    mock_delete_gke.return_value.delete.assert_called_once_with()
+
+
 @pytest.mark.parametrize(
     "is_gcp,direct_runner,config_override,image_tag",
     (
@@ -386,9 +360,11 @@ def test_delete_job(
         (True, True, "klio-job2.yaml", "foobar"),
     ),
 )
+@pytest.mark.parametrize("is_job_dir_override", (False, True))
 def test_run_job(
     runner,
     mocker,
+    tmpdir,
     is_gcp,
     direct_runner,
     image_tag,
@@ -396,6 +372,8 @@ def test_run_job(
     config_file,
     mock_get_git_sha,
     mock_klio_config,
+    mock_error_stackdriver_logger_metrics,
+    is_job_dir_override,
 ):
     mock_run = mocker.patch.object(cli.job_commands.run.RunPipeline, "run")
     mock_run.return_value = 0
@@ -425,6 +403,98 @@ def test_run_job(
             ],
         },
     }
+    job_dir_override = None
+    if is_job_dir_override:
+        temp_dir = tmpdir.mkdir("testing12345")
+        job_dir_override = str(temp_dir)
+
+    mock_klio_config.setup(
+        config_data, config_file, config_override, job_dir_override
+    )
+
+    cli_inputs = ["job", "run"]
+    if image_tag:
+        cli_inputs.extend(["--image-tag", image_tag])
+    if direct_runner:
+        cli_inputs.append("--direct-runner")
+    if config_override:
+        cli_inputs.extend(["--config-file", config_override])
+    if job_dir_override:
+        cli_inputs.extend(["--job-dir", job_dir_override])
+
+    exp_image_tag = image_tag or mock_get_git_sha.return_value
+    if config_override:
+        exp_image_tag = "{}-{}".format(exp_image_tag, config_override)
+
+    result = runner.invoke(cli.main, cli_inputs)
+
+    core_testing.assert_execution_success(result)
+    assert "" == result.output
+
+    mock_klio_config.assert_calls()
+    mock_run.assert_called_once_with()
+    mock_get_git_sha.assert_called_once_with(
+        mock_klio_config.meta.job_dir, image_tag
+    )
+    mock_error_stackdriver_logger_metrics.assert_called_once_with(
+        mock_klio_config.klio_config, direct_runner
+    )
+
+
+@pytest.mark.parametrize(
+    "direct_runner,config_override,image_tag",
+    (
+        (True, None, None),
+        (False, None, None),
+        (True, None, "foobar"),
+        (True, "klio-job2.yaml", None),
+        (True, "klio-job2.yaml", "foobar"),
+    ),
+)
+def test_run_job_gke(
+    runner,
+    mocker,
+    tmpdir,
+    direct_runner,
+    image_tag,
+    config_override,
+    config_file,
+    mock_get_git_sha,
+    mock_klio_config,
+    mock_error_stackdriver_logger_metrics,
+):
+    mock_run_gke = mocker.patch.object(gke_commands.RunPipelineGKE, "run")
+    mock_run_gke.return_value = 0
+    mock_run = mocker.patch.object(cli.job_commands.run.RunPipeline, "run")
+    mock_run.return_value = 0
+
+    config_data = {
+        "job_name": "test-job",
+        "pipeline_options": {
+            "worker_harness_container_image": "gcr.register.io/squad/feature",
+            "project": "test-project",
+            "region": "boonies",
+            "staging_location": "gs://somewhere/over/the/rainbow",
+            "temp_location": "gs://somewhere/over/the/rainbow",
+            "runner": "DirectGKERunner",
+        },
+        "job_config": {
+            "inputs": [
+                {
+                    "topic": "foo-topic",
+                    "subscription": "foo-sub",
+                    "data_location": "foo-input-location",
+                }
+            ],
+            "outputs": [
+                {
+                    "topic": "foo-topic-output",
+                    "data_location": "foo-output-location",
+                }
+            ],
+        },
+    }
+
     mock_klio_config.setup(config_data, config_file, config_override)
 
     cli_inputs = ["job", "run"]
@@ -441,13 +511,24 @@ def test_run_job(
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
-    mock_run.assert_called_once_with()
+    mock_klio_config.assert_calls()
+
+    if direct_runner:
+        mock_run.assert_called_once_with()
+        mock_run_gke.assert_not_called()
+    else:
+        mock_run_gke.assert_called_once_with()
+        mock_run.assert_not_called()
+
     mock_get_git_sha.assert_called_once_with(
         mock_klio_config.meta.job_dir, image_tag
     )
+    mock_error_stackdriver_logger_metrics.assert_called_once_with(
+        mock_klio_config.klio_config, direct_runner
+    )
 
 
 def test_run_job_raises(
@@ -460,7 +541,7 @@ def test_run_job_raises(
     mock_get_git_sha,
 ):
     mock_run = mocker.patch.object(cli.job_commands.run.RunPipeline, "run")
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
 
     cli_inputs = ["job", "run", "--image-tag", "foobar"]
 
@@ -485,8 +566,6 @@ def test_stop_job(
     mock_stop,
     mocker,
     config_file,
-    mock_get_config_job_dir,
-    patch_os_getcwd,
     pipeline_config_dict,
     mock_klio_config,
 ):
@@ -518,7 +597,7 @@ def test_stop_job(
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     mock_klio_config.assert_calls()
@@ -527,6 +606,35 @@ def test_stop_job(
     )
 
 
+def test_stop_job_gke(
+    runner, mocker, mock_klio_config,
+):
+    config_data = {
+        "job_name": "test-job",
+        "version": 1,
+        "pipeline_options": {
+            "project": "test-project",
+            "runner": "DirectGKERunner",
+        },
+        "job_config": {},
+    }
+    mock_klio_config.setup(config_data, None, None)
+
+    mock_stop_gke = mocker.patch.object(
+        cli.job_commands.gke, "StopPipelineGKE"
+    )
+
+    cli_inputs = ["job", "stop"]
+
+    result = runner.invoke(cli.main, cli_inputs)
+
+    core_testing.assert_execution_success(result)
+    assert "" == result.output
+
+    mock_klio_config.assert_calls()
+    mock_stop_gke.assert_called_once_with(mock_klio_config.meta.job_dir)
+
+
 @pytest.mark.parametrize(
     "is_gcp,direct_runner,config_override,image_tag",
     (
@@ -549,6 +657,7 @@ def test_deploy_job(
     pipeline_config_dict,
     mock_get_git_sha,
     mock_klio_config,
+    mock_error_stackdriver_logger_metrics,
 ):
     mock_run = mocker.patch.object(cli.job_commands.run.RunPipeline, "run")
     mock_run.return_value = 0
@@ -592,7 +701,7 @@ def test_deploy_job(
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     mock_get_git_sha.assert_called_once_with(patch_os_getcwd, image_tag)
@@ -601,6 +710,9 @@ def test_deploy_job(
         "test-job", "test-project", "us-central1", "cancel"
     )
     mock_run.assert_called_once_with()
+    mock_error_stackdriver_logger_metrics.assert_called_once_with(
+        mock_klio_config.klio_config, direct_runner
+    )
 
 
 def test_deploy_job_raises(
@@ -614,7 +726,7 @@ def test_deploy_job_raises(
     mock_get_git_sha,
 ):
     mock_run = mocker.patch.object(cli.job_commands.run.RunPipeline, "run")
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
 
     cli_inputs = ["job", "deploy", "--image-tag", "foobar"]
 
@@ -695,16 +807,16 @@ def test_test_job(
             ],
         },
     }
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
     # deepcopy since KlioConfig will pop keys
     mock_get_config.return_value = config_data
     conf = kconfig.KlioConfig(copy.deepcopy(config_data))
-    mock_klio_config = mocker.patch.object(cli_utils.config, "KlioConfig")
+    mock_klio_config = mocker.patch.object(core_utils.config, "KlioConfig")
     mock_klio_config.return_value = conf
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     exp_image_tag = image_tag or mock_get_git_sha.return_value
@@ -718,7 +830,9 @@ def test_test_job(
         mock_get_git_sha.assert_not_called()
 
     mock_get_config.assert_called_once_with(conf_override or config_file)
-    mock_klio_config.assert_called_once_with(config_data)
+    mock_klio_config.assert_called_once_with(
+        config_data, raw_overrides=(), raw_templates=()
+    )
     exp_docker_runtime_config = cli.DockerRuntimeConfig(
         image_tag=exp_image_tag,
         force_build=False,
@@ -757,7 +871,7 @@ def test_verify(
         patch_os_getcwd,
         conf_override or config_file,
     )
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
 
     config = {
         "job_name": "klio-job-name",
@@ -779,7 +893,7 @@ def test_verify(
         cli_inputs.extend(["--config-file", conf_override])
 
     result = runner.invoke(cli.main, cli_inputs)
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     mock_get_config_job_dir.assert_called_once_with(None, conf_override)
     mock_get_config.assert_called_once_with(conf_override or config_file)
@@ -832,7 +946,7 @@ def test_collect_profiling_data(
     }
     if not gcs_location:
         config["pipeline_options"] = {"profile_location": "gs://foo"}
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
     mock_get_config.return_value = config
 
     exp_exit_code = 0
@@ -902,7 +1016,7 @@ def test_collect_profiling_data_raises(
         "job_config": {"events": {}, "data": {}},
         "pipeline_options": {},
     }
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
     mock_get_config.return_value = config
 
     cli_inputs = ["job", "profile", "collect-profiling-data"]
@@ -1034,7 +1148,7 @@ def test_profile_memory(runner, mocker, minimal_mock_klio_config):
 
     result = runner.invoke(cli.profile_memory, [])
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     exp_kwargs = {
         "image_tag": None,
@@ -1060,7 +1174,7 @@ def test_profile_memory_per_line(runner, mocker, minimal_mock_klio_config):
     mock_profile = mocker.patch.object(cli, "_profile")
     result = runner.invoke(cli.profile_memory_per_line, [])
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     exp_kwargs = {
         "get_maximum": False,
@@ -1085,7 +1199,7 @@ def test_profile_cpu(runner, mocker, minimal_mock_klio_config):
 
     result = runner.invoke(cli.profile_cpu, [])
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     exp_kwargs = {
         "interval": 0.1,
@@ -1112,7 +1226,7 @@ def test_profile_timeit(
 
     result = runner.invoke(cli.profile_timeit, [])
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
 
     exp_kwargs = {
         "iterations": 10,
@@ -1201,7 +1315,7 @@ def test_publish(
 
     result = runner.invoke(cli.main, cli_inputs)
 
-    assert_execution_success(result)
+    core_testing.assert_execution_success(result)
     assert "" == result.output
 
     mock_publish.assert_called_once_with(
@@ -1324,7 +1438,7 @@ def test_publish_raises_non_klio_config(
     mock_publish = mocker.patch.object(
         cli.message_commands.publish, "publish_messages"
     )
-    mock_get_config = mocker.patch.object(config_utils, "get_config_by_path")
+    mock_get_config = mocker.patch.object(core_utils, "get_config_by_path")
 
     cli_inputs = ["message", "publish", "deadb33f", "--non-klio"]
 
