@@ -13,66 +13,13 @@
 # limitations under the License.
 #
 
-import os
 import subprocess
 
-from unittest import mock
-
 import pytest
 
 from klio_cli.utils import cli_utils
 
 
-@pytest.mark.parametrize(
-    "image",
-    (
-        "dataflow.gcr.io/v1beta3/python",
-        "dataflow.gcr.io/v1beta3/python-base",
-        "dataflow.gcr.io/v1beta3/python-fnapi",
-    ),
-)
-def test_warn_if_py2_job(image, patch_os_getcwd, mocker):
-    dockerfile = (
-        '## -*- docker-image-name: "gcr.io/foo/bar" -*-\n'
-        "FROM {image}:1.2.3\n"
-        'LABEL maintainer "me@example.com"\n'
-    ).format(image=image)
-
-    m_open = mock.mock_open(read_data=dockerfile)
-    mock_open = mocker.patch("klio_cli.utils.cli_utils.open", m_open)
-
-    warn_msg = (
-        "Python 2 support in Klio is deprecated. "
-        "Please upgrade to Python 3.5+"
-    )
-    with pytest.warns(UserWarning, match=warn_msg):
-        cli_utils.warn_if_py2_job(patch_os_getcwd)
-
-    exp_read_file = os.path.join(patch_os_getcwd, "Dockerfile")
-    mock_open.assert_called_once_with(exp_read_file, "r")
-
-
-@pytest.mark.parametrize("has_from_line", (True, False))
-def test_warn_if_py2_job_no_warn(has_from_line, patch_os_getcwd, mocker):
-    from_line = "\n"
-    if has_from_line:
-        from_line = "FROM dataflow.gcr.io/v1beta3/python36-fnapi:1.2.3\n"
-
-    dockerfile = (
-        '## -*- docker-image-name: "gcr.io/foo/bar" -*-\n'
-        + from_line
-        + 'LABEL maintainer "me@example.com"\n'
-    )
-
-    m_open = mock.mock_open(read_data=dockerfile)
-    mock_open = mocker.patch("klio_cli.utils.cli_utils.open", m_open)
-
-    cli_utils.warn_if_py2_job(patch_os_getcwd)
-
-    exp_read_file = os.path.join(patch_os_getcwd, "Dockerfile")
-    mock_open.assert_called_once_with(exp_read_file, "r")
-
-
 @pytest.mark.parametrize("job_dir", (None, "/foo/bar/klio-job.yaml"))
 def test_get_git_sha(mocker, job_dir):
     check_output_mock = mocker.patch.object(subprocess, "check_output")
@@ -138,43 +85,74 @@ def test_validate_dataflow_runner_config(
         cli_utils.validate_dataflow_runner_config(mock_klio_cfg)
 
 
-@pytest.mark.parametrize("direct_runner", [False, True])
-def test_is_direct_runner(mocker, monkeypatch, direct_runner):
+@pytest.mark.parametrize(
+    "config_runner,direct_runner,exp_is_direct,exp_mock_validate_df",
+    (
+        ("direct", True, True, False),
+        ("direct", False, True, False),
+        ("DirectRunner", True, True, False),
+        ("DirectRunner", False, True, False),
+        ("dataflow", True, True, False),
+        ("dataflow", False, False, True),
+        ("DataflowRunner", True, True, False),
+        ("DataflowRunner", False, False, True),
+        ("DirectGKERunner", True, True, False),
+        ("DirectGKERunner", False, False, False),
+    ),
+)
+def test_is_direct_runner(
+    mocker,
+    monkeypatch,
+    config_runner,
+    direct_runner,
+    exp_is_direct,
+    exp_mock_validate_df,
+):
     mock_klio_cfg = mocker.Mock()
+    mock_klio_cfg.pipeline_options.runner = config_runner
     mock_validate_df_config = mocker.Mock()
     monkeypatch.setattr(
         cli_utils, "validate_dataflow_runner_config", mock_validate_df_config
     )
 
-    if not direct_runner:
-        assert mock_validate_df_config.called_once_with(mock_klio_cfg)
+    act_resp = cli_utils.is_direct_runner(mock_klio_cfg, direct_runner)
 
+    assert exp_is_direct == act_resp
+    if exp_mock_validate_df:
+        mock_validate_df_config.assert_called_once_with(mock_klio_cfg)
+
+
+@pytest.mark.parametrize(
+    "sd_config",
+    (
+        {"stackdriver_logger": True},
+        {"stackdriver_logger": {"timer_unit": "s"}},
+    ),
+)
+def test_error_stackdriver_logger_metrics_raises(sd_config, mocker, caplog):
+    mock_klio_config = mocker.Mock()
+    mock_klio_config.pipeline_options.runner = "DataflowRunner"
+    mock_klio_config.job_config.metrics = {"stackdriver_logger": sd_config}
+
+    with pytest.raises(SystemExit):
+        cli_utils.error_stackdriver_logger_metrics(mock_klio_config, False)
+
+    assert 1 == len(caplog.records)
     assert (
-        cli_utils.is_direct_runner(mock_klio_cfg, direct_runner)
-        == direct_runner
+        "The Stackdriver log-based metric client has been deprecated"
+        in caplog.records[0].message
     )
 
 
 @pytest.mark.parametrize(
-    "job_dir,conf_file",
-    (
-        (None, None),
-        (None, "klio-job2.yaml"),
-        ("foo/bar", None),
-        ("foo/bar", "klio-job2.yaml"),
-    ),
+    "metrics_conf",
+    ({"stackdriver_logger": False}, {"native": {"timer_unit": "s"}}, {},),
 )
-def test_get_config_job_dir(job_dir, conf_file, patch_os_getcwd):
-    exp_job_dir = patch_os_getcwd
-    if job_dir:
-        exp_job_dir = os.path.abspath(os.path.join(patch_os_getcwd, job_dir))
-    exp_conf_file = conf_file or os.path.join(exp_job_dir, "klio-job.yaml")
-    if job_dir and conf_file:
-        exp_conf_file = os.path.join(job_dir, conf_file)
+def test_error_stackdriver_logger_metrics(metrics_conf, mocker, caplog):
+    mock_klio_config = mocker.Mock()
+    mock_klio_config.pipeline_options.runner = "DataflowRunner"
+    mock_klio_config.job_config.metrics = metrics_conf
 
-    ret_job_dir, ret_conf_file = cli_utils.get_config_job_dir(
-        job_dir, conf_file
-    )
+    cli_utils.error_stackdriver_logger_metrics(mock_klio_config, False)
 
-    assert exp_job_dir == ret_job_dir
-    assert exp_conf_file == ret_conf_file
+    assert 0 == len(caplog.records)
